import { defineComponent, onBeforeUnmount, onMounted, ref } from "vue";
import { any, array } from "vue-types";

import { MatController } from "@/controller/MatController";
import { List } from "@queenjs/ui";
import PanelCard from "../components/PanelCard";
import PanelGroup from "../components/PanelGroup";
import MatItem from "./MatItem";

export default defineComponent({
  props: {
    sourceData: array<any>().isRequired,
    controller: any<MatController>().isRequired,
  },
  setup(props) {
    const compRef = ref();

    onMounted(() => props.controller.initScroll(compRef.value));

    onBeforeUnmount(() => props.controller.unbindScroll());

    return () => {
      const { controller, sourceData } = props;

      return (
        <PanelCard title="部件库" onSave={controller.saveMatch} ref={compRef}>
          {sourceData.map((matsGroup: MatsMatchComp, index: number) => {
            const isGroupActive = controller.isGroupActive(matsGroup);

            return (
              <PanelGroup
                key={index}
                class="JSDrop cursor-pointer"
                type="asset3d.mat"
                title={matsGroup.cusName || matsGroup.name}
                tools={["rename", "lock", "clear"]}
                target={matsGroup}
                active={isGroupActive}
                onAction={(key) => controller.matGroupAction(key, matsGroup)}
                onClick={() => controller.switchGroupIndex(matsGroup)}
              >
                <List data={matsGroup.matIds || []} columns={5} gap="5px">
                  {{
                    item: (matId: string, index: number) => {
                      const currentMats = controller.findMatById(matId);
                      return (
                        <MatItem
                          key={matId}
                          mat={currentMats}
                          active={index == matsGroup.index}
                          onClick={() =>
                            controller.switchGroupMatIndex(matsGroup, index)
                          }
                          onAction={(key: string) =>
                            controller.matItemAction(
                              key,
                              matsGroup,
                              index,
                              currentMats
                            )
                          }
                        />
                      );
                    },
                    empty: () => (
                      <div class="my-30px text-12px text-center text-gray-500">
                        暂无数据
                      </div>
                    ),
                  }}
                </List>
              </PanelGroup>
            );
          })}
          <div class="mt-20px text-12px text-center text-gray-400">
            没有更多了
          </div>
        </PanelCard>
      );
    };
  },
});
